@@ -23,16 +23,19 @@ that didn't really belong anywhere. */
 #ifdef HAVE_BSTRING_H
 #include <bstring.h>
 #endif
-#ifdef MSW
+#ifdef _WIN32
 #include <io.h>
 #include <fcntl.h>
 #include <process.h>
 #include <winsock.h>
 #include <windows.h>
+# ifdef _MSC_VER
 typedef int pid_t;
+# endif
 typedef int socklen_t;
 #define EADDRINUSE WSAEADDRINUSE
 #endif
+
 #include <stdarg.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -87,7 +90,6 @@ struct _socketreceiver
 
 extern char *pd_version;
 extern int sys_guisetportnumber;
-extern char sys_font[]; /* tb: typeface */
 
 static int sys_nfdpoll;
 static t_fdpoll *sys_fdpoll;
@@ -150,13 +152,15 @@ double sys_getrealtime(void)
 #endif
 }
 
+extern int sys_nosleep;
+
 static int sys_domicrosleep(int microsec, int pollem)
 {
     struct timeval timout;
     int i, didsomething = 0;
     t_fdpoll *fp;
     timout.tv_sec = 0;
-    timout.tv_usec = microsec;
+    timout.tv_usec = (sys_nosleep ? 0 : microsec);
     if (pollem)
     {
         fd_set readset, writeset, exceptset;
@@ -246,12 +250,14 @@ static void sys_huphandler(int n)
 void sys_setalarm(int microsec)
 {
     struct itimerval gonzo;
+    int sec = (int)(microsec/1000000);
+    microsec %= 1000000;
 #if 0
-    fprintf(stderr, "timer %d\n", microsec);
+    fprintf(stderr, "timer %d:%d\n", sec, microsec);
 #endif
     gonzo.it_interval.tv_sec = 0;
     gonzo.it_interval.tv_usec = 0;
-    gonzo.it_value.tv_sec = 0;
+    gonzo.it_value.tv_sec = sec;
     gonzo.it_value.tv_usec = microsec;
     if (microsec)
         sys_signal(SIGALRM, sys_alarmhandler);
@@ -412,12 +418,12 @@ void socketreceiver_free(t_socketreceiver *x)
 static int socketreceiver_doread(t_socketreceiver *x)
 {
     char messbuf[INBUFSIZE], *bp = messbuf;
-    int indx;
+    int indx, first = 1;
     int inhead = x->sr_inhead;
     int intail = x->sr_intail;
     char *inbuf = x->sr_inbuf;
-    if (intail == inhead) return (0);
-    for (indx = intail; indx != inhead; indx = (indx+1)&(INBUFSIZE-1))
+    for (indx = intail; first || (indx != inhead);
+        first = 0, (indx = (indx+1)&(INBUFSIZE-1)))
     {
             /* if we hit a semi that isn't preceeded by a \, it's a message
             boundary.  LATER we should deal with the possibility that the
@@ -538,6 +544,8 @@ void socketreceiver_read(t_socketreceiver *x, int fd)
                     if (x->sr_socketreceivefn)
                         (*x->sr_socketreceivefn)(x->sr_owner, inbinbuf);
                     else binbuf_eval(inbinbuf, 0, 0, 0);
+                    if (x->sr_inhead == x->sr_intail)
+                        break;
                 }
             }
         }
@@ -844,7 +852,8 @@ void glob_watchdog(t_pd *dummy)
 
 #define FIRSTPORTNUM 5400
 
-static int defaultfontshit[] = {
+#define MAXFONTS 21
+static int defaultfontshit[MAXFONTS] = {
     8, 5, 9, 10, 6, 10, 12, 7, 13, 14, 9, 17, 16, 10, 19, 24, 15, 28,
         24, 15, 28};
 #define NDEFAULTFONT (sizeof(defaultfontshit)/sizeof(*defaultfontshit))
@@ -1011,6 +1020,74 @@ int sys_startgui(const char *guidir)
 
 
 #ifdef UNISTD
+        if (!sys_guicmd)
+        {
+#ifdef __APPLE__
+            char *homedir = getenv("HOME"), filename[250];
+            struct stat statbuf;
+                /* first look for Wish bundled with and renamed "Pd" */
+            sprintf(filename, "%s/../../MacOS/Pd", guidir);
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            if (!homedir || strlen(homedir) > 150)
+                goto nohomedir;
+                /* Look for Wish in user's Applications.  Might or might
+                not be names "Wish Shell", and might or might not be
+                in "Utilities" subdir. */
+            sprintf(filename,
+                "%s/Applications/Utilities/Wish shell.app/Contents/MacOS/Wish Shell",
+                    homedir);
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            sprintf(filename,
+                "%s/Applications/Utilities/Wish.app/Contents/MacOS/Wish",
+                    homedir);
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            sprintf(filename,
+                "%s/Applications/Wish shell.app/Contents/MacOS/Wish Shell",
+                    homedir);
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            sprintf(filename,
+                "%s/Applications/Wish.app/Contents/MacOS/Wish",
+                    homedir);
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+        nohomedir:
+                /* Perform the same search among system applications. */
+            strcpy(filename, 
+                "/usr/bin/wish");
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            strcpy(filename, 
+                "/Applications/Utilities/Wish Shell.app/Contents/MacOS/Wish Shell");
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            strcpy(filename, 
+                "/Applications/Utilities/Wish.app/Contents/MacOS/Wish");
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            strcpy(filename, 
+                "/Applications/Wish Shell.app/Contents/MacOS/Wish Shell");
+            if (stat(filename, &statbuf) >= 0)
+                goto foundit;
+            strcpy(filename, 
+                "/Applications/Wish.app/Contents/MacOS/Wish");
+        foundit:
+            sprintf(cmdbuf, "\"%s\" %s/pd.tk %d\n", filename, guidir, portno);
+#else
+            sprintf(cmdbuf,
+                "TCL_LIBRARY=\"%s/tcl/library\" TK_LIBRARY=\"%s/tk/library\" \
+                 \"%s/pd-gui\" %d\n",
+                 sys_libdir->s_name, sys_libdir->s_name, guidir, portno);
+#endif
+            sys_guicmd = cmdbuf;
+        }
+
+        if (sys_verbose) 
+            fprintf(stderr, "%s", sys_guicmd);
+
         childpid = fork();
         if (childpid < 0)
         {
@@ -1038,71 +1115,10 @@ int sys_startgui(const char *guidir)
                 }
             }
 #endif
-            if (!sys_guicmd)
-            {
-#ifdef __APPLE__
-                char *homedir = getenv("HOME"), filename[250];
-                struct stat statbuf;
-                    /* first look for Wish bundled with and renamed "Pd" */
-                sprintf(filename, "%s/../../MacOS/Pd", guidir);
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                if (!homedir || strlen(homedir) > 150)
-                    goto nohomedir;
-                    /* Look for Wish in user's Applications.  Might or might
-                    not be names "Wish Shell", and might or might not be
-                    in "Utilities" subdir. */
-                sprintf(filename,
-                    "%s/Applications/Utilities/Wish shell.app/Contents/MacOS/Wish Shell",
-                        homedir);
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                sprintf(filename,
-                    "%s/Applications/Utilities/Wish.app/Contents/MacOS/Wish",
-                        homedir);
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                sprintf(filename,
-                    "%s/Applications/Wish shell.app/Contents/MacOS/Wish Shell",
-                        homedir);
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                sprintf(filename,
-                    "%s/Applications/Wish.app/Contents/MacOS/Wish",
-                        homedir);
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-            nohomedir:
-                    /* Perform the same search among system applications. */
-                strcpy(filename, 
-                    "/Applications/Utilities/Wish Shell.app/Contents/MacOS/Wish Shell");
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                strcpy(filename, 
-                    "/Applications/Utilities/Wish.app/Contents/MacOS/Wish");
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                strcpy(filename, 
-                    "/Applications/Wish Shell.app/Contents/MacOS/Wish Shell");
-                if (stat(filename, &statbuf) >= 0)
-                    goto foundit;
-                strcpy(filename, 
-                    "/Applications/Wish.app/Contents/MacOS/Wish");
-            foundit:
-                sprintf(cmdbuf, "\"%s\" %s/pd.tk %d\n", filename, guidir, portno);
-#else
-                sprintf(cmdbuf,
-"TCL_LIBRARY=\"%s/tcl/library\" TK_LIBRARY=\"%s/tk/library\" \
- \"%s/pd-gui\" %d\n",
-                    sys_libdir->s_name, sys_libdir->s_name, guidir, portno);
-#endif
-                sys_guicmd = cmdbuf;
-            }
-            if (sys_verbose) fprintf(stderr, "%s", sys_guicmd);
             execl("/bin/sh", "sh", "-c", sys_guicmd, (char*)0);
             perror("pd: exec");
             _exit(1);
-        }
+       }
 #endif /* UNISTD */
 
 #ifdef MSW
@@ -1153,8 +1169,8 @@ int sys_startgui(const char *guidir)
             to make it timeshare with the rest of the system.  (Version
             0.33P2 : if there's no GUI, the watchdog pinging is done
             from the scheduler idle routine in this process instead.) */
-
         int pipe9[2], watchpid;
+
         if (pipe(pipe9) < 0)
         {
             seteuid(getuid());      /* lose setuid priveliges */
@@ -1228,7 +1244,7 @@ int sys_startgui(const char *guidir)
         sys_guisock = accept(xsock, (struct sockaddr *) &server, 
             (socklen_t *)&len);
 #ifdef OOPS
-        close(xsock);
+        sys_closesocket(xsock);
 #endif
         if (sys_guisock < 0) sys_sockerror("accept");
         if (sys_verbose)
@@ -1248,8 +1264,8 @@ int sys_startgui(const char *guidir)
 #endif
          sys_get_audio_apis(buf);
          sys_get_midi_apis(buf2);
-         sys_vgui("pdtk_pd_startup {%s} %s %s {%s}\n", pd_version, buf, buf2,
-                                  sys_font); 
+         sys_vgui("pdtk_pd_startup {%s} %s %s {%s} %s\n", pd_version, buf, buf2, 
+                  sys_font, sys_fontweight); 
     }
     return (0);
 
@@ -1283,7 +1299,7 @@ void glob_quit(void *dummy)
     sys_vgui("exit\n");
     if (!sys_nogui)
     {
-        close(sys_guisock);
+        sys_closesocket(sys_guisock);
         sys_rmpollfn(sys_guisock);
     }
     sys_bail(0); 
