@@ -20,6 +20,9 @@
 #include "m_pd.h"
 #include "s_stuff.h"
 #include <stdio.h>
+#ifdef _MSC_VER  /* This is only for Microsoft's compiler, not cygwin, e.g. */
+#define snprintf sprintf_s
+#endif
 
 typedef void (*t_xxx)(void);
 
@@ -35,7 +38,7 @@ a fat binary or an indication of the instruction set. */
 static char sys_dllextent[] = ".b_i386", sys_dllextent2[] = ".pd_freebsd";
 #endif
 #ifdef __linux__
-#ifdef __ia64__
+#ifdef __x86_64__
 static char sys_dllextent[] = ".l_ia64", sys_dllextent2[] = ".pd_linux";
 #else
 static char sys_dllextent[] = ".l_i386", sys_dllextent2[] = ".pd_linux";
@@ -124,7 +127,7 @@ static int sys_do_load_lib(t_canvas *canvas, char *objectname)
     symname[i] = 0;
     if (hexmunge)
     {
-        memmove(symname+6, symname, strlen(symname+1));
+        memmove(symname+6, symname, strlen(symname)+1);
         strncpy(symname, "setup_", 6);
     }
     else strcat(symname, "_setup");
@@ -238,3 +241,37 @@ int sys_load_lib(t_canvas *canvas, char *classname)
     return ok;
 }
 
+int sys_run_scheduler(const char *externalschedlibname,
+    const char *sys_extraflagsstring)
+{
+    typedef int (*t_externalschedlibmain)(const char *);
+    t_externalschedlibmain externalmainfunc;
+    char filename[MAXPDSTRING];
+    snprintf(filename, sizeof(filename), "%s.%s", externalschedlibname,
+        sys_dllextent);
+    sys_bashfilename(filename, filename);
+#ifdef MSW
+    {
+        HINSTANCE ntdll = LoadLibrary(filename);
+        if (!ntdll)
+        {
+            post("%s: couldn't load external scheduler lib ", filename);
+            return (0);
+        }
+        externalmainfunc =
+            (t_externalschedlibmain)GetProcAddress(ntdll, "main");
+    }
+#else
+    {
+        void *dlobj = dlopen(filename, RTLD_NOW | RTLD_GLOBAL);
+        if (!dlobj)
+        {
+            post("%s: %s", filename, dlerror());
+            return (0);
+        }
+        externalmainfunc = (t_externalschedlibmain)dlsym(dlobj,
+            "pd_extern_sched");
+    }
+#endif
+    return((*externalmainfunc)(sys_extraflagsstring));
+}
