@@ -21,6 +21,9 @@
 #include <windows.h>
 #include <winbase.h>
 #endif
+#ifdef _MSC_VER  /* This is only for Microsoft's compiler, not cygwin, e.g. */
+#define snprintf sprintf_s
+#endif
 
 char *pd_version;
 char pd_compiletime[] = __TIME__;
@@ -31,7 +34,7 @@ int sys_argparse(int argc, char **argv);
 void sys_findprogdir(char *progname);
 int sys_startgui(const char *guipath);
 int sys_rcfile(void);
-int m_scheduler(void);
+int m_mainloop(void);
 void sys_addhelppath(char *p);
 #ifdef USEAPI_ALSA
 void alsa_adddev(char *name);
@@ -43,11 +46,11 @@ int sys_noloadbang;
 int sys_nogui;
 int sys_hipriority = -1;    /* -1 = don't care; 0 = no; 1 = yes */
 int sys_guisetportnumber;   /* if started from the GUI, this is the port # */
+int sys_nosleep = 0;  /* skip all "sleep" calls and spin instead */
 
 char *sys_guicmd;
 t_symbol *sys_libdir;
 static t_symbol *sys_guidir;
-t_namelist *sys_externlist;
 static t_namelist *sys_openlist;
 static t_namelist *sys_messagelist;
 static int sys_version;
@@ -58,19 +61,27 @@ int sys_nmidiin = -1;
 int sys_midiindevlist[MAXMIDIINDEV] = {1};
 int sys_midioutdevlist[MAXMIDIOUTDEV] = {1};
 
-char sys_font[100] = "courier"; /* tb: font name */
+char sys_font[100] = 
+#ifdef MSW
+    "Courier";
+#else
+    "Courier";
+#endif
+char sys_fontweight[] = "bold  "; /* currently only used for iemguis */
 static int sys_main_srate;
 static int sys_main_advance;
+static int sys_main_callback;
 static int sys_listplease;
 
 int sys_externalschedlib;
 char sys_externalschedlibname[MAXPDSTRING];
 int sys_extraflags;
 char sys_extraflagsstring[MAXPDSTRING];
-
+int sys_run_scheduler(const char *externalschedlibname,
+    const char *sys_extraflagsstring);
 
     /* here the "-1" counts signify that the corresponding vector hasn't been
-    specified in command line arguments; sys_open_audio will detect this
+    specified in command line arguments; sys_set_audio_settings will detect it
     and fill things in. */
 static int sys_nsoundin = -1;
 static int sys_nsoundout = -1;
@@ -88,7 +99,7 @@ int* get_sys_main_advance() { return &sys_main_advance; }
 double* get_sys_time_per_dsp_tick() { return &sys_time_per_dsp_tick; }
 int* get_sys_schedblocksize() { return &sys_schedblocksize; }
 double* get_sys_time() { return &sys_time; }
-float* get_sys_dacsr() { return &sys_dacsr; }
+t_float* get_sys_dacsr() { return &sys_dacsr; }
 int* get_sys_sleepgrain() { return &sys_sleepgrain; }
 int* get_sys_schedadvance() { return &sys_schedadvance; }
 
@@ -169,7 +180,7 @@ static void openit(const char *dirname, const char *filename)
     char dirbuf[MAXPDSTRING], *nameptr;
     int fd = open_via_path(dirname, filename, "", dirbuf, &nameptr,
         MAXPDSTRING, 0);
-    if (fd)
+    if (fd >= 0)
     {
         close (fd);
         glob_evalfile(0, gensym(nameptr), gensym(dirbuf));
@@ -285,35 +296,15 @@ int sys_main(int argc, char **argv)
     if (sys_startgui(sys_guidir->s_name))       /* start the gui */
         return(1);
     if (sys_externalschedlib)
-    {
-#ifdef MSW
-        typedef int (*t_externalschedlibmain)(char *);
-        t_externalschedlibmain externalmainfunc;
-        HINSTANCE ntdll;
-        char filename[MAXPDSTRING];
-
-        sprintf(filename, "%s.dll", sys_externalschedlibname);
-        sys_bashfilename(filename, filename);
-        ntdll = LoadLibrary(filename);
-        if (!ntdll)
-        {
-              post("%s: couldn't load external scheduler lib ", filename);
-              return (0);
-        }
-        externalmainfunc = (t_externalschedlibmain)GetProcAddress(ntdll,
-                                                                  "main");
-        return((*externalmainfunc)(sys_extraflagsstring));
-#else
-        return (0);
-#endif
-    }
+        return (sys_run_scheduler(sys_externalschedlibname,
+            sys_extraflagsstring));
     else
     {
             /* open audio and MIDI */
         sys_reopen_midi();
         sys_reopen_audio();
             /* run scheduler until it quits */
-        return (m_scheduler());
+        return (m_mainloop());
     }
 }
 
@@ -382,8 +373,9 @@ static char *(usagemessage[]) = {
 "-helppath <path> -- add to help file search path\n",
 "-open <file>     -- open file(s) on startup\n",
 "-lib <file>      -- load object library(s)\n",
-"-font <n>        -- specify default font size in points\n",
-"-typeface <name> -- specify default font (default: courier)\n",
+"-font-size <n>     -- specify default font size in points\n",
+"-font-face <name>  -- specify default font\n",
+"-font-weight <name>-- specify default font weight (normal or bold)\n",
 "-verbose         -- extra printout on startup and when searching for files\n",
 "-version         -- don't run Pd; just print out which version it is \n",
 "-d <n>           -- specify debug level\n",
@@ -398,6 +390,7 @@ static char *(usagemessage[]) = {
 "-rt or -realtime -- use real-time priority\n",
 "-nrt             -- don't use real-time priority\n",
 #endif
+"-nosleep         -- spin, don't sleep (may lower latency on multi-CPUs)\n",
 };
 
 static void sys_parsedevlist(int *np, int *vecp, int max, char *str)
@@ -579,6 +572,11 @@ int sys_argparse(int argc, char **argv)
             sys_main_advance = atoi(argv[1]);
             argc -= 2; argv += 2;
         }
+        else if (!strcmp(*argv, "-callback"))
+        {
+            sys_main_callback = 1;
+            argc--; argv++;
+        }
         else if (!strcmp(*argv, "-blocksize"))
         {
             sys_setblocksize(atoi(argv[1]));
@@ -586,7 +584,7 @@ int sys_argparse(int argc, char **argv)
         }
         else if (!strcmp(*argv, "-sleepgrain") && (argc > 1))
         {
-            sys_sleepgrain = 1000 * atoi(argv[1]);
+            sys_sleepgrain = 1000 * atof(argv[1]);
             argc -= 2; argv += 2;
         }
         else if (!strcmp(*argv, "-nodac"))
@@ -737,21 +735,28 @@ int sys_argparse(int argc, char **argv)
             sys_externlist = namelist_append_files(sys_externlist, argv[1]);
             argc -= 2; argv += 2;
         }
-        else if (!strcmp(*argv, "-font") && argc > 1)
+        else if ((!strcmp(*argv, "-font-size") || !strcmp(*argv, "-font"))
+            && argc > 1)
         {
             sys_defaultfont = sys_nearestfontsize(atoi(argv[1]));
             argc -= 2;
             argv += 2;
         }
-                /* tb: font name { */
-        else if (!strcmp(*argv, "-typeface") && argc > 1)
+        else if ((!strcmp(*argv, "-font-face") || !strcmp(*argv, "-typeface"))
+            && argc > 1)
         {
             strncpy(sys_font,*(argv+1),sizeof(sys_font)-1);
             sys_font[sizeof(sys_font)-1] = 0;
             argc -= 2;
             argv += 2;
         }
-                /* } tb */
+        else if (!strcmp(*argv, "-font-weight") && argc > 1)
+        {
+            strncpy(sys_fontweight,*(argv+1),sizeof(sys_fontweight)-1);
+            sys_fontweight[sizeof(sys_fontweight)-1] = 0;
+            argc -= 2;
+            argv += 2;
+        }
         else if (!strcmp(*argv, "-verbose"))
         {
             sys_verbose++;
@@ -807,14 +812,16 @@ int sys_argparse(int argc, char **argv)
         else if (!strcmp(*argv, "-schedlib"))
         {
             sys_externalschedlib = 1;
-            strcpy(sys_externalschedlibname, argv[1]);
+            strncpy(sys_externalschedlibname, argv[1],
+                sizeof(sys_externalschedlibname) - 1);
             argv += 2;
             argc -= 2;
         }
         else if (!strcmp(*argv, "-extraflags"))
         {
             sys_extraflags = 1;
-            strcpy(sys_extraflagsstring, argv[1]);
+            strncpy(sys_extraflagsstring, argv[1],
+                sizeof(sys_extraflagsstring) - 1);
             argv += 2;
             argc -= 2;
         }
@@ -830,6 +837,11 @@ int sys_argparse(int argc, char **argv)
             argc--; argv++;
         }
 #endif
+        else if (!strcmp(*argv, "-nosleep"))
+        {
+            sys_nosleep = 1;
+            argc--; argv++;
+        }
         else if (!strcmp(*argv, "-soundindev") ||
             !strcmp(*argv, "-audioindev"))
         {
@@ -893,7 +905,7 @@ static void sys_afterargparse(void)
     int i;
     int naudioindev, audioindev[MAXAUDIOINDEV], chindev[MAXAUDIOINDEV];
     int naudiooutdev, audiooutdev[MAXAUDIOOUTDEV], choutdev[MAXAUDIOOUTDEV];
-    int nchindev, nchoutdev, rate, advance;
+    int nchindev, nchoutdev, rate, advance, callback;
     int nmidiindev = 0, midiindev[MAXMIDIINDEV];
     int nmidioutdev = 0, midioutdev[MAXMIDIOUTDEV];
             /* add "extra" library to path */
@@ -929,7 +941,7 @@ static void sys_afterargparse(void)
             else are the default.  Overwrite them with any results
             of argument parsing, and store them again. */
     sys_get_audio_params(&naudioindev, audioindev, chindev,
-        &naudiooutdev, audiooutdev, choutdev, &rate, &advance);
+        &naudiooutdev, audiooutdev, choutdev, &rate, &advance, &callback);
     if (sys_nchin >= 0)
     {
         nchindev = sys_nchin;
@@ -975,8 +987,11 @@ static void sys_afterargparse(void)
         advance = sys_main_advance;
     if (sys_main_srate)
         rate = sys_main_srate;
-    sys_open_audio(naudioindev, audioindev, nchindev, chindev,
-        naudiooutdev, audiooutdev, nchoutdev, choutdev, rate, advance, 0);
+    if (sys_main_callback)
+        callback = sys_main_callback;
+    sys_set_audio_settings(naudioindev, audioindev, nchindev, chindev,
+        naudiooutdev, audiooutdev, nchoutdev, choutdev, rate, advance, 
+        callback);
     sys_open_midi(nmidiindev, midiindev, nmidioutdev, midioutdev, 0);
 }
 
