@@ -30,6 +30,13 @@
 #include <fcntl.h>
 #include <ctype.h>
 
+#ifdef _LARGEFILE64_SOURCE
+# define open  open64
+# define lseek lseek64
+# define fstat fstat64
+# define stat  stat64
+#endif
+
 t_namelist *sys_externlist;
 t_namelist *sys_searchpath;
 t_namelist *sys_helppath;
@@ -64,21 +71,29 @@ void sys_unbashfilename(const char *from, char *to)
 
 /*******************  Utility functions used below ******************/
 
-/* copy until delimiter and return position after delimiter in string */
-/* if it was the last substring, return NULL */
-
-static const char* strtokcpy(char *to, const char *from, int delim)
+/*!
+ * \brief copy until delimiter
+ * 
+ * \arg to destination buffer
+ * \arg to_len destination buffer length
+ * \arg from source buffer
+ * \arg delim string delimiter to stop copying on
+ *
+ * \return position after delimiter in string.  If it was the last
+ *         substring, return NULL.
+ */
+static const char *strtokcpy(char *to, size_t to_len, const char *from, char delim)
 {
-    int size = 0;
+    unsigned int i = 0;
 
-    while (from[size] != (char)delim && from[size] != '\0')
-        size++;
+        for (; i < (to_len - 1) && from[i] && from[i] != delim; i++)
+                to[i] = from[i];
+        to[i] = '\0';
 
-    strncpy(to,from,size);
-    to[size] = '\0';
-    if (from[size] == '\0') return NULL;
-    if (size) return from+size+1;
-    else return NULL;
+        if (i && from[i] != '\0')
+                return from + i + 1;
+
+        return NULL;
 }
 
 /* add a single item to a namelist.  If "allowdup" is true, duplicates
@@ -126,7 +141,7 @@ t_namelist *namelist_append_files(t_namelist *listwas, const char *s)
     npos = s;
     do
     {
-        npos = strtokcpy(temp, npos, SEPARATOR);
+        npos = strtokcpy(temp, sizeof(temp), npos, SEPARATOR);
         if (! *temp) continue;
         nl = namelist_append(nl, temp, 0);
     }
@@ -262,7 +277,7 @@ ends up in the same buffer as dirresult.  Exception:
 if the 'name' starts with a slash or a letter, colon, and slash in MSW,
 there is no search and instead we just try to open the file literally.  */
 
-/* see also canvas_openfile() which, in addition, searches down the
+/* see also canvas_open() which, in addition, searches down the
 canvas-specific path. */
 
 static int do_open_via_path(const char *dir, const char *name,
@@ -515,12 +530,10 @@ void glob_start_path_dialog(t_pd *dummy)
     char buf[MAXPDSTRING];
     int i;
     t_namelist *nl;
-    
-    for (nl = sys_searchpath, i = 0; nl && i < 10; nl = nl->nl_next, i++)
-        sys_vgui("pd_set pd_path%d \"%s\"\n", i, nl->nl_string);
-    for (; i < 10; i++)
-        sys_vgui("pd_set pd_path%d \"\"\n", i);
 
+    sys_gui("global pd_path; set pd_path {}\n");
+    for (nl = sys_searchpath, i = 0; nl; nl = nl->nl_next, i++)
+        sys_vgui("lappend pd_path {%s}\n", nl->nl_string);
     sprintf(buf, "pdtk_path_dialog %%s %d %d\n", sys_usestdpath, sys_verbose);
     gfxstub_new(&glob_pdobject, (void *)glob_start_path_dialog, buf);
 }
@@ -547,12 +560,10 @@ void glob_start_startup_dialog(t_pd *dummy)
     char buf[MAXPDSTRING];
     int i;
     t_namelist *nl;
-    
-    for (nl = sys_externlist, i = 0; nl && i < 10; nl = nl->nl_next, i++)
-        sys_vgui("pd_set pd_startup%d \"%s\"\n", i, nl->nl_string);
-    for (; i < 10; i++)
-        sys_vgui("pd_set pd_startup%d \"\"\n", i);
 
+    sys_gui("global pd_startup; set pd_startup {}\n");
+    for (nl = sys_externlist, i = 0; nl; nl = nl->nl_next, i++)
+        sys_vgui("lappend pd_startup {%s}\n", nl->nl_string);
     sprintf(buf, "pdtk_startup_dialog %%s %d \"%s\"\n", sys_defeatrt,
         sys_flags->s_name);
     gfxstub_new(&glob_pdobject, (void *)glob_start_startup_dialog, buf);
